@@ -1,3 +1,5 @@
+package Rubyish::Attribute;
+use 5.010;
 
 =head1 NAME
 
@@ -5,22 +7,24 @@ Rubyish::Attribute - ruby-like accessor builder: attr_accessor, attr_writer and
 
 =cut
 
-package Rubyish::Attribute;
+use Want;
 
-use Sub::Exporter;
-Sub::Exporter::setup_exporter({ 
-    exports => [qw(attr_accessor attr_writer attr_reader)],
-    groups  => { default => [-all] },
-});
+sub import {
+  my $caller = caller;
+  for (qw(attr_accessor attr_reader attr_writer)) {
+    *{$caller . "::" . $_} = *{$_};
+  }
+  eval qq{package $caller; use PadWalker qw(peek_my);};
+}
 
 
 =head1 VERSION
 
-This document is for version 1.0
+This document is for version 1.1
 
 =cut
 
-our $VERSION = "1.0";
+our $VERSION = "1.1";
 
 =head1 SYNOPSIS
 
@@ -46,6 +50,14 @@ our $VERSION = "1.0";
             bless {}, $class;
         }
 
+        sub rename_as {
+          my ($self, $new_name) = @_;
+          __name__ = $new_name;
+
+          # __name__ is accurately a lvalue subroutine &__name__() which refer to $self->{name}
+          # now it looks like a instance variable.
+        }
+
         1;
     }
    
@@ -72,24 +84,27 @@ Each attribute could be read by getter as showing in synopsis.
 
 =cut
 
+
+sub make_accessor {
+    my $field = shift;
+    return sub {
+        my ($self, $arg) = @_;
+        if ($arg) {
+            $self->{$field} = $arg;
+            $self;
+        }
+        else {
+            $self->{$field};
+        }
+    }
+}
+
 sub attr_accessor {
     no strict;
-
-    my $make_accessor = sub {
-        my $field = shift;
-        return sub {
-            my ($self, $arg) = @_;
-            if ($arg) {
-                $self->{$field} = $arg;
-                $self;
-            } else {
-                $self->{$field};
-            }
-        }
-    };
-
+    my $package = caller;
     for my $field (@_) {
-        *{(caller)[0] . "::" . $field} = $make_accessor->($field);
+        *{"${package}::${field}"} = make_accessor($field);
+        make_instance_vars_accessor($package, $field);
     }
 }
 
@@ -104,24 +119,26 @@ attr_reader create only getter for the class you call it
 
 =cut
 
+sub make_reader {
+    my $field = shift;
+    return sub {
+        my ($self, $arg) = @_;
+        if ($arg) {
+            warn "error - $field is only reader\n";
+            return;             # because no writer
+        }
+        else {
+            $self->{$field};
+        }
+    }
+};
+
 sub attr_reader {
     no strict;
-
-    my $make_reader = sub {
-        my $field = shift;
-        return sub {
-            my ($self, $arg) = @_;
-            if ($arg) {
-                warn "error - $field is only reader\n";
-                return; # because no writer
-            } else {
-                $self->{$field};
-            }
-        }
-    };
-    
+    my $package = caller;
     for my $field (@_) {
-        *{(caller)[0] . "::" . $field} = $make_reader->($field);
+        *{"${package}::${field}"} = make_reader($field);
+        make_instance_vars_accessor($package, $field);
     }
 }
 
@@ -136,35 +153,47 @@ attr_writer create only setter for the class you call it.
 
 =cut
 
+sub make_writer {
+    my $field = shift;
+    return sub {
+        my ($self, $arg) = @_;
+        if ($arg) {
+            $self->{$field} = $arg;
+            $self;
+        }
+        else {
+            warn "error - $field is only writer\n";
+            return;             # because no reader 
+        }
+    }
+}
+
 sub attr_writer {
     no strict;
-
-    my $make_writer = sub {
-        my $field = shift;
-        return sub {
-            my ($self, $arg) = @_;
-            if ($arg) {
-                $self->{$field} = $arg;
-                $self;
-            } else {
-                warn "error - $field is only writer\n";
-                return; # because no reader 
-            }
-        }
-    };
-
+    my $package = caller;
     for my $field (@_) {
-        *{(caller)[0] . "::" . $field} = $make_writer->($field);
+        *{"${package}::${field}"} = make_writer($field);
+        make_instance_vars_accessor($package, $field);
     }
 }
 
+sub make_instance_vars_accessor {
+  no strict;
+  my ($package, $field) = @_;
+  eval qq|package $package;
+    sub __${field}__ : lvalue {
+      \${ peek_my(1)->{\'\$self\'} }->{$field};
+    }
+  |;
+}
+
 =head1 DEPENDENCE
 
-L<Sub::Exporter>
+L<Want>
 
 =head1 SEE ALSO
 
-L<Sub::Exporter>, L<autobox::Core>, L<List::Rubyish>
+L<autobox::Core>, L<List::Rubyish>, L<Class::Accessor::Lvalue>, L<Want>
 
 L<http://ruby-doc.org/core-1.8.7/classes/Module.html#M000423>
 
@@ -192,9 +221,9 @@ please report bugs to <shelling at cpan.org> or <gugod at gugod.org>
 
 =head1 COPYRIGHT & LICENCE 
 
-Copyright © 2008 shelling, gugod, all rights reserved.
+Copyright (C) 2008 shelling, gugod, all rights reserved.
 
-This program is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
+Release under MIT (X11) Lincence.
 
 =cut
 
